unit SOAPImpl;

{ TODO : Cover custom exception classes for Web Services (step 4 in the online tutorial) }

//This server unit implements the invokable interface

interface

uses
  SOAPIntf, InvokeRegistry;

type
  //To implement an invokable class, inherit from
  //InvokeRegistry.TInvokableClass (basically the same as
  //TInterfacedObject, but with virtual constructor so the
  //invocation registry can polymorphically create it)

  //An alternative to inheriting from TInvokableClass is to supply
  //the invocation registry with a method to retrieve instances of
  //your class (as the 2nd argument to RegisterInvokableClass) and
  //you must handle the lifetime management of your class.
  TGeometry = class(TInvokableClass, IGeometry)
  protected
    function SphereVolume(Radius: Double): Double; stdcall;
    function CircleArea(Radius: Double): Double; stdcall;
  end;

implementation

{ TGeometry }

function TGeometry.CircleArea(Radius: Double): Double;
begin
  Result := Pi * Sqr(Radius)
end;

function TGeometry.SphereVolume(Radius: Double): Double;
begin
  Result := (4/3) * Pi * Sqr(Radius) * Radius
end;

//If we weren't inheriting from TInvokableClass, we'd need a factory
//routine to create instances of our class, like this. Note that
//we must take care of lifetime management (or return references to a
//global instance)
//procedure CreateGeometry(out obj: TObject);
//begin
//  obj := TGeometry.Create;
//end;

initialization
  //Registering a class inherited from TInvokableClass
  InvRegistry.RegisterInvokableClass(TGeometry);

  //How to register a class not inherited from TInvokableClass
  //InvRegistry.RegisterInvokableClass(TGeometry, CreateGeometry)
end.
